/* iplog.h of iplog v1.2 -- eric@ojnk.nu */
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <errno.h>
extern int errno;

#include "config.h"

#ifndef DROP_USER
#	define DROP_USER "nobody"
#endif

#ifndef DROP_GROUP
#	define DROP_GROUP "nogroup"
#endif

#ifndef LOGGING_OPTIONS
#	define LOGGING_OPTIONS	LOG_PID
#endif

#ifndef LOGGING_FACILITY
#	define LOGGING_FACILITY LOG_DAEMON
#endif

#ifndef LOGGING_PRIORITY
#	define LOGGING_PRIORITY	LOG_NOTICE
#endif

#define LEVEL	LOGGING_PRIORITY

void fatal(const char *fmt, ...) {
	char *p = malloc(512);
	va_list ap;

	if (p == NULL)
		exit(-1);

	va_start(ap, fmt);
	(void)vsnprintf(p, 512, fmt, ap);
	va_end(p);

	syslog(LOG_WARNING, p);
	free(p);

	exit(-1);
}


void fork_to_background(void) {
	switch (fork()) {
		case -1:
			_exit(0);
		case 0:
			break;
		default:
			_exit(0);
	}

	if (setsid() == -1)
		fatal("setsid: %m");

	chdir("/");
	umask(022);

	(void)close(0); /* stdin */
	(void)close(1); /* stdout */
	(void)close(2); /* stderr */

	return;
}

void drop_privileges(void) {
	struct passwd *pw = NULL;
	struct group *gr = NULL;

	if ((pw = getpwnam(DROP_USER)) == NULL)
		fatal("getpwnam: %m");

	if ((gr = getgrnam(DROP_GROUP)) == NULL)
		fatal("getgrnam: %m");

	if (setregid(gr->gr_gid, gr->gr_gid) < 0)
		fatal("setregid: %m");

	if (initgroups(pw->pw_name, gr->gr_gid) < 0)
		fatal("initgroups: %m");

	if (setreuid(pw->pw_uid, pw->pw_uid) < 0)
		fatal("setreuid: %m");
	
	return;
}
